@@ -14,10 +14,9 @@ import Yesod.Form.Functions
 import Yesod.Form.Fields (boolField)
 import Yesod.Widget (GWidget, whamlet)
 import Yesod.Message (RenderMessage)
-import Yesod.Handler (newIdent, GGHandler)
+import Yesod.Handler (newIdent, GHandler)
 import Text.Blaze (Html)
 import Control.Monad.Trans.Class (lift)
-import Data.Text (pack)
 import Control.Monad.Trans.RWS (get, put, ask)
 import Data.Maybe (fromMaybe)
 import Data.Text.Read (decimal)
@@ -53,7 +52,7 @@ up i = do
         IntCons _ is' -> put is' >> newFormIdent >> return ()
     up $ i - 1
 
-inputList :: (m ~ GGHandler sub master IO, xml ~ GWidget sub master (), RenderMessage master FormMessage)
+inputList :: (m ~ GHandler sub master, xml ~ GWidget sub master (), RenderMessage master FormMessage)
           => Html
           -> ([[FieldView sub master]] -> xml)
           -> (Maybe a -> AForm sub master a)
@@ -79,10 +78,10 @@ inputList label fixXml single mdef = formToAForm $ do
     let count = length vals
     (res, xmls, views) <- liftM fixme $ mapM (withDelete . single) vals
     up 1
-    return (res, FieldView
+    return (res, [FieldView
         { fvLabel = label
         , fvTooltip = Nothing
-        , fvId = pack theId
+        , fvId = theId
         , fvInput = [WHAMLET|
 ^{fixXml views}
 <p>
@@ -94,7 +93,7 @@ inputList label fixXml single mdef = formToAForm $ do
 |]
         , fvErrors = Nothing
         , fvRequired = False
-        })
+        }])
 
 withDelete :: (xml ~ GWidget sub master (), RenderMessage master FormMessage)
            => AForm sub master a
@@ -111,6 +110,7 @@ withDelete af = do
                 , fsTooltip = Nothing
                 , fsName = Just deleteName
                 , fsId = Nothing
+                , fsClass = []
                 } $ Just False
             (res, xml) <- aFormToForm af
             return $ Right (res, xml $ xml2 [])
