@@ -11,6 +11,7 @@ module Yesod.Form.Nic
     ) where
 
 import Yesod.Handler
+import Yesod.Core (Route, ScriptLoadPosition(..), jsLoader, Yesod)
 import Yesod.Form
 import Yesod.Widget
 import Text.HTML.SanitizeXSS (sanitizeBalance)
@@ -18,11 +19,11 @@ import Text.Hamlet (Html, shamlet)
 import Text.Julius (julius)
 import Text.Blaze.Renderer.String (renderHtml)
 import Text.Blaze (preEscapedText)
-import Control.Monad.Trans.Class (lift)
 import Data.Text (Text, pack)
+import qualified Data.Text as T
 import Data.Maybe (listToMaybe)
 
-class YesodNic a where
+class Yesod a => YesodNic a where
     -- | NIC Editor Javascript file.
     urlNicEdit :: a -> Either (Route a) Text
     urlNicEdit _ = Right "http://js.nicedit.com/nicEdit-latest.js"
@@ -30,23 +31,34 @@ class YesodNic a where
 nicHtmlField :: YesodNic master => Field sub master Html
 nicHtmlField = Field
     { fieldParse = return . Right . fmap (preEscapedText . sanitizeBalance) . listToMaybe
-    , fieldView = \theId name val _isReq -> do
+    , fieldView = \theId name theClass val _isReq -> do
         addHtml
 #if __GLASGOW_HASKELL__ >= 700
                 [shamlet|
 #else
                 [$shamlet|
 #endif
-    <textarea id="#{theId}" name="#{name}" .html>#{showVal val}
+    <textarea id="#{theId}" :not (null theClass):class="#{T.intercalate " " theClass}" name="#{name}" .html>#{showVal val}
 |]
         addScript' urlNicEdit
-        addJulius
+        master <- lift getYesod
+        addJulius $
+          case jsLoader master of
+            BottomOfHeadBlocking ->
 #if __GLASGOW_HASKELL__ >= 700
                 [julius|
 #else
                 [$julius|
 #endif
 bkLib.onDomLoaded(function(){new nicEditor({fullPanel:true}).panelInstance("#{theId}")});
+|]
+            _ ->
+#if __GLASGOW_HASKELL__ >= 700
+                [julius|
+#else
+                [$julius|
+#endif
+(function(){new nicEditor({fullPanel:true}).panelInstance("#{theId}")})();
 |]
     }
   where
